@@ -2,7 +2,7 @@
 # -*- Mode: Python; indent-tabs-mode: nil; tab-width: 2; coding: utf-8 -*-
 #
 # This file is part of Déjà Dup.
-# © 2008,2009,2010,2011 Michael Terry <mike@mterry.name>
+# For copyright information, see AUTHORS.
 #
 # Déjà Dup is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,8 +28,7 @@ import ldtp
 import glob
 import re
 import traceback
-
-latest_duplicity = '0.6.13'
+import time
 
 srcdir = "."
 temp_dir = None
@@ -66,7 +65,7 @@ def get_temp_name(extra, make=False):
 # if we're running inside a distcheck for example.  So note that we check for
 # srcdir and use it if available.  Else, default to current directory.
 
-def setup(start = True, args=[''], root_prompt = False):
+def setup(root_prompt = False):
   global cleanup_dirs, cleanup_pids, cleanup_envs, ldtp, latest_duplicity, srcdir
 
   srcdir = environ.get('srcdir')
@@ -80,14 +79,14 @@ def setup(start = True, args=[''], root_prompt = False):
   environ['LANG'] = 'C'
   environ['DEJA_DUP_TESTING'] = '1'
 
-  extra_paths = ':'.join(['%s/../%s' % (builddir, x) for x in ['deja-dup', 'preferences', 'applet', 'monitor']]) + ':'
+  extra_paths = ':'.join(['%s/../%s' % (builddir, x) for x in ['deja-dup', 'preferences', 'monitor']]) + ':'
   extra_pythonpaths = ''
   
   version = None
   if 'DEJA_DUP_TEST_VERSION' in environ:
     version = environ['DEJA_DUP_TEST_VERSION']
   if version is None:
-    version = latest_duplicity
+    version = 'system'
   if version != 'system':
     os.system('%s/build-duplicity %s' % (srcdir, version))
     duproot = '%s/duplicity/duplicity-%s' % (srcdir, version)
@@ -108,7 +107,7 @@ def setup(start = True, args=[''], root_prompt = False):
   environ['XDG_CACHE_HOME'] = get_temp_name('cache')
   environ['XDG_CONFIG_HOME'] = get_temp_name('config')
   environ['XDG_DATA_HOME'] = get_temp_name('share')
-  environ['XDG_DATA_DIRS'] = "%s:%s" % (environ['XDG_DATA_HOME'], environ['XDG_DATA_DIRS'])
+  environ['XDG_DATA_DIRS'] = "%s:/usr/share/" % environ['XDG_DATA_HOME']
   output = subprocess.Popen(['dbus-launch'], stdout=subprocess.PIPE).communicate()[0]
   lines = output.split('\n')
   for line in lines:
@@ -119,8 +118,11 @@ def setup(start = True, args=[''], root_prompt = False):
           os.environ[parts[0]] = parts[1]
 
   # Shutdown ldtpd so that it will restart and pick up new dbus environment
-  os.system('pkill -f ldtpd\\.main; sleep 1')
-  ldtp.getwindowlist()
+  #os.system('pkill -f ldtpd\\.main; sleep 1')
+  #ldtp.getwindowlist()/reload(ldtp)
+
+  process = subprocess.Popen(['/usr/lib/d-conf/dconf-service'], stdout=subprocess.PIPE)
+  cleanup_pids += [process.pid]
 
   environ['PYTHONPATH'] = extra_pythonpaths + (environ['PYTHONPATH'] if 'PYTHONPATH' in environ else '')
   environ['PATH'] = extra_paths + environ['PATH']
@@ -130,31 +132,22 @@ def setup(start = True, args=[''], root_prompt = False):
   
   os.system('mkdir -p %s' % environ['XDG_CONFIG_HOME'])
   os.system('mkdir -p %s/glib-2.0/schemas/' % environ['XDG_DATA_HOME'])
-  
-  # Make sure file chooser has txtLocation
-  os.system('mkdir -p %s/gtk-2.0' % environ['XDG_CONFIG_HOME'])
-  os.system('echo [Filechooser Settings] > "%s/gtk-2.0/gtkfilechooser.ini"' % environ['XDG_CONFIG_HOME'])
-  os.system('echo LocationMode=filename-entry >> "%s/gtk-2.0/gtkfilechooser.ini"' % environ['XDG_CONFIG_HOME'])
 
   # Now install default schema into our temporary config dir
-  if os.system('cp %s/../data/org.gnome.DejaDup.gschema.xml %s/glib-2.0/schemas/ && glib-compile-schemas %s/glib-2.0/schemas/' % (builddir, environ['XDG_DATA_HOME'], environ['XDG_DATA_HOME'])):
+  schemas = glob.glob("/usr/share/glib-2.0/schemas/org.gtk.*")
+  schemas.append("%s/../data/org.gnome.DejaDup.gschema.xml" % builddir)
+  if os.system('cp %s %s/glib-2.0/schemas/ && glib-compile-schemas %s/glib-2.0/schemas/' % (' '.join(schemas), environ['XDG_DATA_HOME'], environ['XDG_DATA_HOME'])):
     raise Exception('Could not install settings schema')
 
   # Copy interface files into place as well
   os.system("mkdir -p %s/deja-dup/ui" % environ['XDG_DATA_HOME'])
   os.system("cp %s/../data/ui/* %s/deja-dup/ui" % (srcdir, environ['XDG_DATA_HOME']))
 
+  set_settings_value("location-mode", "'filename-entry'",
+                     root='org.gtk.Settings.FileChooser:/org/gtk/settings/file-chooser/')
+
   set_settings_value("root-prompt", 'true' if root_prompt else 'false')
 
-  #daemon_env = subprocess.Popen(['gnome-keyring-daemon'], stdout=subprocess.PIPE).communicate()[0].strip()
-  #daemon_env = daemon_env.split('\n')
-  #for line in daemon_env:
-  #  bits = line.split('=')
-  #  os.environ[bits[0]] = bits[1]
-
-  if start:
-    start_deja_dup(args)
-
 def cleanup(success):
   global temp_dir, cleanup_dirs, cleanup_mounts, cleanup_pids, cleanup_envs
   for d in cleanup_mounts:
@@ -180,28 +173,38 @@ def cleanup(success):
     os.system('bash -c "echo -e \'\e[31mFAILED\e[0m\'"')
     sys.exit(1)
 
-def set_settings_value(key, value, schema = None):
+def set_settings_value(key, value, schema = None, root = None):
+  if root is None:
+    root = 'org.gnome.DejaDup'
   if schema:
-    schema = 'org.gnome.DejaDup.' + schema
+    fullschema = root + '.' + schema
   else:
-    schema = 'org.gnome.DejaDup'
-  cmd = ['gsettings', 'set', schema, key, value]
+    fullschema = root
+  cmd = ['gsettings', 'set', fullschema, key, value]
   sp = subprocess.Popen(cmd, stdout=subprocess.PIPE)
   sp.communicate()
   if sp.returncode:
     raise Exception('Could not set key %s to %s' % (key, value))
+  # make sure everything is set correctly
+  setval = get_settings_value(key, schema=schema, root=root)
+  assert setval == value, "%s != %s" % (setval, value)
 
-def get_settings_value(key, schema = None):
+def get_settings_value(key, schema = None, root = None):
+  if root is None:
+    root = 'org.gnome.DejaDup'
   if schema:
-    schema = 'org.gnome.DejaDup.' + schema
+    fullschema = root + '.' + schema
   else:
-    schema = 'org.gnome.DejaDup'
-  cmd = ['gsettings', 'get', schema, key]
+    fullschema = root
+  cmd = ['gsettings', 'get', fullschema, key]
   sp = subprocess.Popen(cmd, stdout=subprocess.PIPE)
   pout = sp.communicate()[0]
+  if sp.returncode:
+    raise Exception('Could not get key %s' % (key))
+  pout = pout.split('@as ', 1)[-1] # chop off leading @as if any (on empty arrays)
   return pout.strip()
 
-def start_deja_dup(args=[], executable='deja-dup', waitfor='frmDéjàDup', debug=False):
+def start_deja_dup(args=[], executable='deja-dup', waitfor='frmBackup', debug=False):
   # Rather than running debug, it's sometimes more effective to run
   # "./interactive shell" and then run gdb directly
   cmd = [executable] + args
@@ -216,11 +219,13 @@ def start_deja_dup(args=[], executable='deja-dup', waitfor='frmDéjàDup', debug
            '--error-exitcode=1', '--suppressions=valgrind.sup'] + cmd
   if debug:
     cmd = ['gnome-terminal', '-x', 'gdb', '-ex', 'run'] + cmd
-  environ['G_SLICE'] = 'always-malloc,debug-blocks'
+  # FIXME gtk3/gail3 seem problematic with always-malloc
+  #environ['G_SLICE'] = 'always-malloc,debug-blocks'
+  #environ['G_SLICE'] = 'debug-blocks'
   environ['G_DEBUG'] = 'gc-friendly' if not environ.get('G_DEBUG') else environ['G_DEBUG'] + ',gc-friendly'
   subprocess.Popen(cmd)
   if waitfor is not None:
-    ldtp.waittillguiexist(waitfor)
+    waitforgui(waitfor)
 
 def create_vol_config(dest='/'):
   if dest is None:
@@ -259,8 +264,10 @@ def create_mount(path=None, mtype='ext', size=20):
   return mount_dir
 
 def quit():
-  if ldtp.guiexist('frmDéjàDup'):
-    ldtp.selectmenuitem('frmDéjàDup', 'mnuBackup;mnuClose')
+  if ldtp.guiexist('frmBackup'):
+    ldtp.closewindow('frmBackup')
+  if ldtp.guiexist('frmBackUp'):
+    ldtp.closewindow('frmBackUp')
 
 def run(method):
   success = False
@@ -295,8 +302,6 @@ def get_manifest_date(filename):
 def list_manifests(dest='local'):
   destdir = get_temp_name(dest)
   files = sorted(glob.glob('%s/*.manifest*' % destdir), key=get_manifest_date)
-  if not files:
-    raise Exception("Expected manifest, found none")
   files = filter(lambda x: x.count('duplicity-full') == 0 or x.count('.to.') == 0, files) # don't get the in-between manifests
   return (destdir, files)
 
@@ -309,6 +314,7 @@ def num_manifests(mtype=None, dest='local'):
 def last_manifest(dest='local'):
   '''Returns last backup manifest (directory, filename) pair'''
   destdir, files = list_manifests(dest)
+  assert len(files) > 0
   latest = files[-1]
   return (destdir, latest)
 
@@ -351,22 +357,36 @@ def guivisible(frm, obj, prefix=False):
   states = ldtp.getallstates(frm, obj)
   return ldtp.state.VISIBLE in states
 
-def wait_for_encryption(dlg, obj, max_count, prefix=False):
+def wait_for_encryption(dlg, encrypt):
+  count = 0
+  max_count = 10
+  while count < max_count:
+    if guivisible(dlg, 'txtEncryptionpassword'):
+      if encrypt:
+        ldtp.settextvalue(dlg, 'txtEncryptionpassword', 'test')
+        if guivisible(dlg, 'txtConfirmpassword'):
+          ldtp.settextvalue(dlg, 'txtConfirmpassword', 'test')
+      else:
+        ldtp.click(dlg, 'rbtnAllowrestoringwithoutapassword')
+      ldtp.click(dlg, 'btnContinue')
+      break
+    time.sleep(1)
+    count += 1
+  assert count < max_count, max_count
+
+def wait_for_finish(dlg, obj, max_count, prefix=False):
   count = 0
   while count < max_count:
-    if guiexist(dlg, obj, prefix):
+    if obj is not None and guivisible(dlg, obj, prefix):
       break
-    if ldtp.guiexist('dlgAllowaccess'):
-      ldtp.click('dlgAllowaccess', 'btnDeny')
-    if ldtp.guiexist(dlg, 'txtEncryptionpassword'):
-      ldtp.settextvalue(dlg, 'txtEncryptionpassword', 'test')
-      if ldtp.guiexist(dlg, 'txtConfirmpassword'):
-        ldtp.settextvalue(dlg, 'txtConfirmpassword', 'test')
-      ldtp.click(dlg, 'btnContinue')
-    ldtp.wait(2)
-    remap(dlg)
+    elif obj is None and not ldtp.guiexist(dlg):
+      break
+    ldtp.wait(1)
     count += 1
-  assert guivisible(dlg, obj, prefix)
+  assert count < max_count, max_count
+
+def waitforgui(frm):
+  assert ldtp.waittillguiexist(frm)
 
 def remap(frm):
   ldtp.wait(1) # sometimes (only for newer versions?) ldtp needs a second to catch its breath
@@ -381,144 +401,168 @@ def set_file_list(dlg, obj, addObj, removeObj, files):
     except:
       break
 
+  # FIXME compiz and modal windows tend to not work with ldtp
+  set_settings_value('include-list' if obj == 'tblIncludeList' else 'exclude-list',
+                     '[%s]' % ', '.join(["'%s'" % os.path.abspath(x) for x in files]))
+  return
+
   # Add new items
   for f in files:
     ldtp.click(dlg, addObj)
-    assert ldtp.waittillguiexist('dlgChoosefolders')
+    waitforgui('dlgChoosefolders')
     # Make sure path ends in '/'
     if f[-1] != '/':
       f += '/'
+    ldtp.selectlastrow('dlgChoosefolders', 'tblPlaces') # must switch away from Recent Files view to get txtLocation
     ldtp.settextvalue('dlgChoosefolders', 'txtLocation', f)
     ldtp.click('dlgChoosefolders', 'btnOpen')
     ldtp.wait(1) # let dialog close
 
-def walk_restore_prefs(dlg, backend = None, encrypt = None, dest = None):
+def walk_prefs(backend, dest, includes, excludes):
   if backend == 'file':
+    ldtp.selectrow('frmBackup', 'tblCategories', 'Storage')
+
     if dest is None:
       dest = get_temp_name('local')
       os.system('mkdir -p %s' % dest)
     elif dest[0] != '/' and dest.find(':') == -1:
       dest = os.getcwd()+'/'+dest
 
-    ldtp.comboselect(dlg, 'cboAmazonS3', 'Local Folder')
-    remap(dlg)
-    ldtp.settextvalue(dlg, 'txt0', dest) # FIXME txt0 is bad name
+    ldtp.comboselect('frmBackup', 'cboLocation', 'Local Folder')
+    ldtp.settextvalue('frmBackup', 'txt0', dest) # FIXME txt0 is bad name
+    ldtp.wait(1) # without this, sometimes ldtp moves so fast, deja-dup doesn't notice dest
 
-  if encrypt is not None:
-    chklabel = 'chkEncryptbackupfiles'
-    if not ldtp.guiexist(dlg, chklabel):
-      chklabel = 'chkBackupfilesareencrypted'
-    if encrypt:
-      ldtp.check(dlg, chklabel)
-    else:
-      ldtp.uncheck(dlg, chklabel)
-
-  ldtp.click(dlg, 'btnForward')
-  remap(dlg)
-
-def walk_backup_prefs(dlg, backend = None, encrypt = None, dest = None, includes = [], excludes = []):
-  walk_restore_prefs(dlg, backend, encrypt, dest)
-  ldtp.wait(1) # give ldtp a second
-
-  # FIXME: bad names
-  set_file_list(dlg, 'tbl1', 'btnAdd1', 'btnRemove1', includes)
-  set_file_list(dlg, 'tbl0', 'btnAdd', 'btnRemove', excludes)
-  ldtp.click(dlg, 'btnForward')
-  remap(dlg)
+  ldtp.selectrow('frmBackup', 'tblCategories', 'Folders')
+  if includes is not None:
+    set_file_list('frmBackup', 'tblIncludeList', 'btnIncludeListAdd', 'btnIncludeListRemove', includes)
+  if excludes is not None:
+    set_file_list('frmBackup', 'tblExcludeList', 'btnExcludeListAdd', 'btnExcludeListRemove', excludes)
 
 def strip_obj_name(obj):
-  return obj.replace("_", "").replace(".", "")
+  return obj.replace("_", "").replace(".", "") if obj is not None else None
 
-def backup_simple(finish=True, error=None, timeout=400, backend = None, encrypt = None, dest = None, includes = [], excludes = [], add_srcdir=True):
+def backup_simple(finish=True, error=None, timeout=400, backend = None,
+                  encrypt = True, dest = None, includes = None, excludes = None,
+                  add_srcdir = True):
   global srcdir
   if add_srcdir:
-    includes = [os.path.join(srcdir, f) for f in includes]
-    excludes = [os.path.join(srcdir, f) for f in excludes]
+    if includes is not None:
+      includes = [os.path.join(srcdir, f) for f in includes]
+    if excludes is not None:
+      excludes = [os.path.join(srcdir, f) for f in excludes]
+
+  start_deja_dup(executable='deja-dup-preferences')
+
+  if guivisible('frmBackup', 'btnJustshowmybackupsettings'):
+    ldtp.click('frmBackup', 'btnJustshowmybackupsettings')
+
+  if backend is not None or includes is not None or excludes is not None:
+    walk_prefs(backend=backend, dest=dest, includes=includes,
+               excludes=excludes)
+
+  ldtp.selectrow('frmBackup', 'tblCategories', 'Overview')
+  ldtp.click('frmBackup', 'btnBackUpNow')
+  waitforgui('frmBackUp')
+
+  if encrypt is not None:
+    wait_for_encryption('frmBackUp', encrypt)
 
-  ldtp.click('frmDéjàDup', 'btnBackUp…')
-  assert ldtp.waittillguiexist('dlgBackUp')
-  remap('dlgBackUp')
-  if guivisible('dlgBackUp', 'lblPreferences'):
-    walk_backup_prefs('dlgBackUp', backend=backend, encrypt=encrypt, dest=dest, includes=includes, excludes=excludes)
-  ldtp.click('dlgBackUp', 'btnBackUp')
-  remap('dlgBackUp')
   if finish:
-    if not error:
-      error = 'lblYourfilesweresuccessfullybackedup'
-      wait_for_encryption('dlgBackUp', error, timeout)
-    else:
-      error = strip_obj_name(error)
-      wait_for_encryption('dlgBackUp', error, timeout, prefix=True)
-    ldtp.click('dlgBackUp', 'btnClose')
-    ldtp.waittillguinotexist('dlgBackUp')
+    error = strip_obj_name(error)
+    wait_for_finish('frmBackUp', error, timeout, prefix=True)
 
-def restore_simple(path, date=None, backend = None, encrypt = None, dest = None):
-  ldtp.click('frmDéjàDup', 'btnRestore…')
-  assert ldtp.waittillguiexist('dlgRestore')
-  remap('dlgRestore')
-  if ldtp.guiexist('dlgRestore', 'lblPreferences'):
-    walk_restore_prefs('dlgRestore', backend=backend, encrypt=encrypt, dest=dest)
-  wait_for_encryption('dlgRestore', 'lblRestorefromWhen?', 200)
+def walk_restore_prefs(dlg, backend, dest):
+  if backend == 'file':
+    ldtp.comboselect(dlg, 'cboLocation', 'Local Folder')
+    ldtp.settextvalue(dlg, 'txt0', dest) # FIXME txt0 is bad name
+    ldtp.wait(1) # without this, sometimes ldtp moves so fast, deja-dup doesn't notice dest
+
+def restore_simple(path, date=None, backend = None, encrypt=True, dest = None):
+  start_deja_dup(executable='deja-dup-preferences')
+
+  if guivisible('frmBackup', 'btnIwanttorestorefilesfromapreviousbackup…'):
+    ldtp.click('frmBackup', 'btnIwanttorestorefilesfromapreviousbackup…')
+  else:
+    ldtp.selectrow('frmBackup', 'tblCategories', 'Overview')
+    ldtp.click('frmBackup', 'btnRestore…')
+
+  waitforgui('frmRestore')
+
+  if backend is not None:
+    walk_restore_prefs('frmRestore', backend, dest)  
+  ldtp.click('frmRestore', 'btnForward')
+
+  wait_for_finish('frmRestore', 'lblRestorefromWhen?', 200)
   if date:
-    ldtp.comboselect('dlgRestore', 'cboDate', date)
-  ldtp.click('dlgRestore', 'btnForward')
-  remap('dlgRestore')
-  ldtp.click('dlgRestore', 'rbtnRestoretospecificfolder')
-  ldtp.comboselect('dlgRestore', 'cboRestorefolder', 'Other...')
-  assert ldtp.waittillguiexist('dlgChoosedestinationforrestoredfiles')
+    ldtp.comboselect('frmRestore', 'cboDate', date)
+  ldtp.click('frmRestore', 'btnForward')
+
+  ldtp.click('frmRestore', 'rbtnRestoretospecificfolder')
+  ldtp.comboselect('frmRestore', 'cboRestorefolder', 'Other...')
+  waitforgui('dlgChoosedestinationforrestoredfiles')
   # Make sure path ends in '/'
   if path[-1] != '/':
     path += '/'
+  ldtp.selectlastrow('dlgChoosedestinationforrestoredfiles', 'tblPlaces') # must switch away from Recent Files view to get txtLocation
   ldtp.settextvalue('dlgChoosedestinationforrestoredfiles', 'txtLocation', path)
   ldtp.click('dlgChoosedestinationforrestoredfiles', 'btnOpen')
   ldtp.wait(1) # give the combo a second to settle
-  ldtp.click('dlgRestore', 'btnForward')
+  ldtp.click('frmRestore', 'btnForward')
   ldtp.wait(1) # give the dlg a second to settle
-  ldtp.click('dlgRestore', 'btnRestore')
-  assert ldtp.waittillguiexist('dlgRestore', 'lblYourfilesweresuccessfullyrestored', 400)
-  assert guivisible('dlgRestore', 'lblYourfilesweresuccessfullyrestored')
-  ldtp.click('dlgRestore', 'btnClose')
-  ldtp.waittillguinotexist('dlgRestore')
+  ldtp.click('frmRestore', 'btnRestore')
 
-def restore_specific(files, path, date=None, backend = None, encrypt = None, dest = None):
+  if encrypt is not None:
+    wait_for_encryption('frmRestore', encrypt)
+
+  wait_for_finish('frmRestore', 'lblYourfilesweresuccessfullyrestored', 400)
+  ldtp.click('frmRestore', 'btnClose')
+  ldtp.waittillguinotexist('frmRestore')
+
+def restore_specific(files, path, date=None, backend = None, encrypt = True, dest = None):
   global srcdir
   files = [os.path.join(srcdir, f) for f in files]
   args = ['--restore'] + files
-  start_deja_dup(args=args, waitfor='dlgRestore')
-  remap('dlgRestore')
-  if ldtp.guiexist('dlgRestore', 'lblPreferences'):
-    walk_restore_prefs('dlgRestore', backend=backend, encrypt=encrypt, dest=dest)
-  wait_for_encryption('dlgRestore', 'lblRestorefromWhen?', 200)
+  start_deja_dup(args=args, waitfor='frmRestore')
+
+  if backend is not None:
+    walk_restore_prefs('frmRestore', backend=backend, dest=dest)
+  ldtp.click('frmRestore', 'btnForward')
+
+  wait_for_finish('frmRestore', 'lblRestorefromWhen?', 200)
   if date:
-    ldtp.comboselect('dlgRestore', 'cboDate', date)
-  ldtp.click('dlgRestore', 'btnForward')
-  ldtp.click('dlgRestore', 'btnRestore')
+    ldtp.comboselect('frmRestore', 'cboDate', date)
+  ldtp.click('frmRestore', 'btnForward')
+  ldtp.click('frmRestore', 'btnRestore')
+
+  if encrypt is not None:
+    wait_for_encryption('frmRestore', encrypt)
+
   if len(files) == 1:
     lbl = 'lblYourfilewassuccessfullyrestored'
   else:
     lbl = 'lblYourfilesweresuccessfullyrestored'
-  assert ldtp.waittillguiexist('dlgRestore', lbl)
-  assert guivisible('dlgRestore', lbl)
-  ldtp.click('dlgRestore', 'btnClose')
+  wait_for_finish('frmRestore', lbl, 400)
+
+  ldtp.click('frmRestore', 'btnClose')
+  ldtp.waittillguinotexist('frmRestore')
 
 def restore_missing(files, path):
   args = ['--restore-missing', path]
-  start_deja_dup(args=args, waitfor='dlgRestore')
-  remap('dlgRestore')
-  wait_for_encryption('dlgRestore', 'lblScanningfinished', 200)
+  start_deja_dup(args=args, waitfor='frmRestore')
+  wait_for_finish('frmRestore', 'lblScanningfinished', 200)
   for f in files:
-    index = ldtp.gettablerowindex('dlgRestore', 'tbl0', f)
+    index = ldtp.gettablerowindex('frmRestore', 'tbl0', f)
     if index != -1:
-      ldtp.checkrow('dlgRestore', 'tbl0', index)
-  ldtp.click('dlgRestore', 'btnForward')
-  ldtp.click('dlgRestore', 'btnRestore')
+      ldtp.checkrow('frmRestore', 'tbl0', index)
+  ldtp.click('frmRestore', 'btnForward')
+  ldtp.click('frmRestore', 'btnRestore')
+  wait_for_encryption('frmRestore', True)
   if len(files) == 1:
     lbl = 'lblYourfilewassuccessfullyrestored'
   else:
     lbl = 'lblYourfilesweresuccessfullyrestored'
-  assert ldtp.waittillguiexist('dlgRestore', lbl)
-  assert guivisible('dlgRestore', lbl)
-  ldtp.click('dlgRestore', 'btnClose')
+  wait_for_finish('frmRestore', lbl, 200)
+  ldtp.click('frmRestore', 'btnClose')
 
 def file_equals(path, contents):
   f = open(path)
