@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: nil; tab-width: 2 -*- */
 /*
     This file is part of Déjà Dup.
-    © 2008–2010 Michael Terry <mike@mterry.name>
+    For copyright information, see AUTHORS.
 
     Déjà Dup is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,14 +26,7 @@
 extern GObject *hacks_status_icon_make_app_indicator (GtkMenu *menu);
 extern void hacks_status_icon_close_app_indicator (GObject *icon);
 
-extern GtkComboBox *hacks_combo_box_text_new();
-extern char *hacks_combo_box_get_text(GtkComboBox *box);
-extern long hacks_window_get_xid(GdkWindow *win);
-extern int hacks_widget_get_allocated_width(GtkWidget *w);
-extern void hacks_widget_destroy(GtkWidget *w);
-extern void hacks_quit_on_destroy(GtkWidget *w);
-extern void hacks_get_natural_size(GtkWidget *w, GtkRequisition *req);
-
+extern gboolean hacks_unity_present(void);
 extern GObject *hacks_unity_get_entry(void);
 extern void hacks_unity_entry_show_progress(GObject *entry, gboolean show);
 extern void hacks_unity_entry_set_progress(GObject *entry, gdouble percent);
